@@ -242,7 +242,7 @@ impl PublicKey {
             encode::set(
                 encode::sequence((
                     oid::AT_COMMON_NAME.encode(),
-                    PublicKeyCn(self).encode(),
+                    PublicKeyCn(self.key_identifier()).encode(),
                 ))
             )
         )
@@ -263,19 +263,19 @@ impl PublicKey {
 
 /// Value encoder for a public key as a common name.
 ///
-/// This type encodes the bits of a public key in a printable string as a
-/// sequence of hex digits as suggested as one option for subject names in
+/// This type encodes the key identifier of a public key in a printable string
+/// as a sequence of hex digits as suggested as one option for subject names in
 /// RPKI certificates by section 8 of [RFC 6487].
 ///
 /// [RFC 6487]: https://tools.ietf.org/html/rfc6487
-#[derive(Clone, Debug)]
-pub struct PublicKeyCn<'a>(&'a PublicKey);
+#[derive(Clone, Copy, Debug)]
+pub struct PublicKeyCn(KeyIdentifier);
 
-impl<'a> PrimitiveContent for PublicKeyCn<'a> {
+impl PrimitiveContent for PublicKeyCn {
     const TAG: Tag = Tag::PRINTABLE_STRING;
 
     fn encoded_len(&self, _mode: Mode) -> usize {
-        self.0.bits.octet_len() * 2
+        self.0.as_slice().len() * 2
     }
 
     fn write_encoded<W: io::Write>(
@@ -283,10 +283,7 @@ impl<'a> PrimitiveContent for PublicKeyCn<'a> {
         _mode: Mode, 
         target: &mut W
     ) -> Result<(), io::Error> {
-        for ch in self.0.bits.octets() {
-            target.write_all(&hex::encode_u8(ch))?
-        }
-        Ok(())
+        target.write_all(&self.0.into_hex())
     }
 }
 
