@@ -1,6 +1,6 @@
 //! The object identifiers used in this crate.
 //!
-//! This module collects all the object indentifiers used at various places
+//! This module collects all the object identifiers used at various places
 //! in this crate in one central place. They are public so you can refer to
 //! them should that ever become necessary.
 
@@ -27,7 +27,7 @@ pub const SHA256_WITH_RSA_ENCRYPTION: ConstOid
 
 /// [RFC 5480](https://tools.ietf.org/html/rfc5480) `ecPublicKey`.
 ///
-/// Indentifies public keys for ellipic curve cryptography.
+/// Identifies public keys for elliptic curve cryptography.
 pub const EC_PUBLIC_KEY: ConstOid = Oid(&[42, 134, 72, 206, 61, 2, 1]);
 
 /// [RFC 5480](https://tools.ietf.org/html/rfc5480) `secp256r1`.
@@ -75,6 +75,8 @@ pub const CT_RPKI_MANIFEST: ConstOid
     = Oid(&[42, 134, 72, 134, 247, 13, 1, 9, 16, 1, 26]);
 pub const CT_RESOURCE_TAGGED_ATTESTATION: ConstOid
     = Oid(&[42, 134, 72, 134, 247, 13, 1, 9, 16, 1, 36]);
+pub const CT_ASPA: ConstOid
+    = Oid(&[42, 134, 72, 134, 247, 13, 1, 9, 16, 1, 49]);
 
 pub const KP_BGPSEC_ROUTER: ConstOid
     = Oid(&[43, 6, 1, 5, 5, 7, 3, 30]);
