@@ -92,13 +92,13 @@ impl IpResources {
                         if v4.is_some() {
                             xerr!(return Err(decode::Malformed.into()));
                         }
-                        v4 = Some(Self::take_from(cons)?);
+                        v4 = Some(Self::take_from(cons, AddressFamily::Ipv4)?);
                     }
                     AddressFamily::Ipv6 => {
                         if v6.is_some() {
                             xerr!(return Err(decode::Malformed.into()));
                         }
-                        v6 = Some(Self::take_from(cons)?);
+                        v6 = Some(Self::take_from(cons, AddressFamily::Ipv6)?);
                     }
                 }
                 Ok(())
@@ -112,7 +112,8 @@ impl IpResources {
 
     /// Takes a single set of  IP resources from a constructed value.
     pub fn take_from<S: decode::Source>(
-        cons: &mut decode::Constructed<S>
+        cons: &mut decode::Constructed<S>,
+        family: AddressFamily,
     ) -> Result<Self, S::Err> {
         cons.take_value(|tag, content| {
             if tag == Tag::NULL {
@@ -120,7 +121,7 @@ impl IpResources {
                 Ok(ResourcesChoice::Inherit)
             }
             else if tag == Tag::SEQUENCE {
-                IpBlocks::parse_content(content)
+                IpBlocks::parse_content(content, family)
                     .map(ResourcesChoice::Blocks)
             }
             else {
@@ -448,24 +449,39 @@ impl IpBlocks {
     pub fn take_from<S: decode::Source>(
         cons: &mut decode::Constructed<S>
     ) -> Result<Self, S::Err> {
-        cons.take_sequence(Self::parse_cons_content)
+        cons.take_sequence(|cons| {
+            // The family is here only for checking the lengths of
+            // bitstrings. Since we don’t care, IPv6 will work.
+            Self::parse_cons_content(cons, AddressFamily::Ipv6)
+        })
+    }
+
+    pub fn take_from_with_family<S: decode::Source>(
+        cons: &mut decode::Constructed<S>,
+        family: AddressFamily
+    ) -> Result<Self, S::Err> {
+        cons.take_sequence(|cons| {
+            Self::parse_cons_content(cons, family)
+        })
     }
 
     /// Parses the content of a AS ID blocks sequence.
     fn parse_content<S: decode::Source>(
-        content: &mut decode::Content<S>
+        content: &mut decode::Content<S>,
+        family: AddressFamily,
     ) -> Result<Self, S::Err> {
         let cons = content.as_constructed()?;
-        Self::parse_cons_content(cons)
+        Self::parse_cons_content(cons, family)
     }
 
     fn parse_cons_content<S: decode::Source>(
-        cons: &mut decode::Constructed<S>
+        cons: &mut decode::Constructed<S>,
+        family: AddressFamily,
     ) -> Result<Self, S::Err> {
         let mut err = None;
 
         let res = iter::repeat_with(||
-            IpBlock::take_opt_from(cons)
+            IpBlock::take_opt_from_with_family(cons, family)
         ).map(|item| {
             match item {
                 Ok(Some(val)) => Some(val),
@@ -677,12 +693,12 @@ impl IpBlock {
         }
     }
 
-    /// Returns an object that sisplays the block as an IPv4 block.
+    /// Returns an object that displays the block as an IPv4 block.
     pub fn display_v4(self) -> DisplayV4Block {
         DisplayV4Block(self)
     }
 
-    /// Returns an object that sisplays the block as an IPv4 block.
+    /// Returns an object that displays the block as an IPv4 block.
     pub fn display_v6(self) -> DisplayV6Block {
         DisplayV6Block(self)
     }
@@ -706,6 +722,27 @@ impl IpBlock {
         })
     }
 
+    pub fn take_opt_from_with_family<S: decode::Source>(
+        cons: &mut decode::Constructed<S>,
+        family: AddressFamily,
+    ) -> Result<Option<Self>, S::Err> {
+        cons.take_opt_value(|tag, content| {
+            if tag == Tag::BIT_STRING {
+                Prefix::parse_content_with_family(
+                    content, family
+                ).map(IpBlock::Prefix)
+            }
+            else if tag == Tag::SEQUENCE {
+                AddressRange::parse_content_with_family(
+                    content, family
+                ).map(IpBlock::Range)
+            }
+            else {
+                xerr!(Err(decode::Malformed.into()))
+            }
+        })
+    }
+
     /// Returns an encoder for the range.
     ///
     /// This encoder will produce a `IPAddressOrRange` value.
@@ -988,13 +1025,46 @@ impl AddressRange {
     fn parse_content<S: decode::Source>(
         content: &mut decode::Content<S>
     ) -> Result<Self, S::Err> {
-        let mut cons = content.as_constructed()?;
+        let cons = content.as_constructed()?;
         Ok(AddressRange {
-            min: Prefix::take_from(&mut cons)?.min(),
-            max: Prefix::take_from(&mut cons)?.max(),
+            min: Prefix::take_from(cons)?.min(),
+            max: Prefix::take_from(cons)?.max(),
         })
     }
 
+    fn parse_content_with_family<S: decode::Source>(
+        content: &mut decode::Content<S>,
+        family: AddressFamily,
+    ) -> Result<Self, S::Err> {
+        let cons = content.as_constructed()?;
+        let min = Self::check_len(Prefix::take_from(cons)?, family)?;
+        let max = Self::check_len(Prefix::take_from(cons)?, family)?;
+        Ok(AddressRange {
+            min: min.min(),
+            max: max.max(),
+        })
+    }
+
+    #[cfg(not(feature = "compat"))]
+    fn check_len(
+        addr: Prefix, family: AddressFamily
+    ) -> Result<Prefix, decode::Error> {
+        if addr.addr_len() > family.max_addr_len() {
+            Err(decode::Malformed)
+        }
+        else {
+            Ok(addr)
+        }
+    }
+
+    #[cfg(feature = "compat")]
+    fn check_len(
+        mut addr: Prefix, family: AddressFamily
+    ) -> Result<Prefix, decode::Error> {
+        addr.len = std::cmp::min(addr.len, family.max_addr_len());
+        Ok(addr)
+    }
+
     /*
     /// Skips over the address range at the beginning of a value.
     fn skip_opt_in<S: decode::Source>(
@@ -1132,12 +1202,12 @@ impl Prefix {
         let len = u8::from_str(&s[sep + 1..])?;
         if addr.is_ipv4() {
             if len > 32 {
-                // XXX Produce an artifical overflow error.
+                // XXX Produce an artificial overflow error.
                 let _ = u8::from_str("256")?;
             }
         }
         else if len > 128 {
-            // XXX Produce an artifical overflow error.
+            // XXX Produce an artificial overflow error.
             let _ = u8::from_str("256")?;
         }
         Ok(Prefix::new(addr, len))
@@ -1148,7 +1218,7 @@ impl Prefix {
         let addr = Ipv4Addr::from_str(&s[..sep])?;
         let len = u8::from_str(&s[sep + 1..])?;
         if len > 32 {
-            // XXX Produce an artifical overflow error.
+            // XXX Produce an artificial overflow error.
             let _ = u8::from_str("256")?;
         }
         Ok(Prefix::new(addr, len))
@@ -1165,7 +1235,7 @@ impl Prefix {
         let addr = Ipv6Addr::from_str(&s[..sep])?;
         let len = u8::from_str(&s[sep + 1..])?;
         if len > 128 {
-            // XXX Produce an artifical overflow error.
+            // XXX Produce an artificial overflow error.
             let _ = u8::from_str("256")?;
         }
         Ok(Prefix::new(addr, len))
@@ -1215,7 +1285,7 @@ impl Prefix {
     /// address, the second element is the largest.
     pub fn range(self) -> (Addr, Addr) {
         // self.addr has all unused bits cleared, so we don’t need to
-        // explicitely do that for min.
+        // explicitly do that for min.
         (self.addr, self.addr.to_max(self.len))
     }
 
@@ -1224,7 +1294,7 @@ impl Prefix {
         self.addr
     }
 
-    /// Returns the largest address convered by the prefix.
+    /// Returns the largest address covered by the prefix.
     pub fn max(self) -> Addr {
         self.addr.to_max(self.addr_len())
     }
@@ -1244,6 +1314,19 @@ impl Prefix {
             &BitString::from_content(content)?
         )?)
     }
+
+    pub fn parse_content_with_family<S: decode::Source>(
+        content: &mut decode::Content<S>,
+        family: AddressFamily,
+    ) -> Result<Self, S::Err> {
+        let res = Self::from_bit_string(
+            &BitString::from_content(content)?
+        )?;
+        if res.addr_len() > family.max_addr_len() {
+            return Err(decode::Malformed.into())
+        }
+        Ok(res)
+    }
 }
 
 
@@ -1308,7 +1391,7 @@ impl encode::PrimitiveContent for Prefix {
 
 //------------ Addr ----------------------------------------------------------
 
-/// An adddress.
+/// An address.
 ///
 /// This can be both an IPv4 and IPv6 address. It keeps the address internally
 /// as a 128 bit unsigned integer. IPv6 address are kept in there in host byte
@@ -1362,6 +1445,7 @@ impl Addr {
     }
 
     /// Returns a byte array for the address.
+    #[allow(clippy::transmute_num_to_bytes)]
     pub fn to_bytes(self) -> [u8; 16] {
         unsafe { mem::transmute(self.0.to_be()) }
     }
@@ -1484,23 +1568,46 @@ impl AddressFamily {
     pub fn take_from<S: decode::Source>(
         cons: &mut decode::Constructed<S>
     ) -> Result<Self, S::Err> {
-        let str = OctetString::take_from(cons)?;
-        let mut octets = str.octets();
+        let octet_string = OctetString::take_from(cons)?;
+        let afi = Self::decode_octet_string(octet_string)?;
+        Ok(afi)
+    }
+
+    pub fn take_opt_from<S: decode::Source>(
+        cons: &mut decode::Constructed<S>
+    ) -> Result<Option<Self>, S::Err> {
+        match OctetString::take_opt_from(cons)? {
+            None => Ok(None),
+            Some(octet_string) => {
+                let afi = Self::decode_octet_string(octet_string)?;
+                Ok(Some(afi))
+            }
+        }
+    }
+
+    pub fn skip_opt_in<S: decode::Source>(
+        cons: &mut decode::Constructed<S>
+    ) -> Result<Option<()>, S::Err> {
+        Self::take_opt_from(cons).map(|opt| opt.map(|_| ()))
+    }
+
+    fn decode_octet_string(octet_string: OctetString) -> Result<AddressFamily, decode::Error> {
+        let mut octets = octet_string.octets();
         let first = match octets.next() {
             Some(first) => first,
-            None => xerr!(return Err(decode::Malformed.into()))
+            None => xerr!(return Err(decode::Malformed))
         };
         let second = match octets.next() {
             Some(second) => second,
-            None => xerr!(return Err(decode::Malformed.into()))
+            None => xerr!(return Err(decode::Malformed))
         };
         if octets.next().is_some() {
-            xerr!(return Err(decode::Malformed.into()))
+            xerr!(return Err(decode::Malformed))
         }
         match (first, second) {
             (0, 1) => Ok(AddressFamily::Ipv4),
             (0, 2) => Ok(AddressFamily::Ipv6),
-            _ => xerr!(Err(decode::Malformed.into())),
+            _ => xerr!(Err(decode::Malformed)),
         }
     }
 
@@ -1512,6 +1619,14 @@ impl AddressFamily {
             }
         )
     }
+
+    /// Returns the maximum prefix length for this family.
+    pub fn max_addr_len(self) -> u8 {
+        match self {
+            AddressFamily::Ipv4 => 32,
+            AddressFamily::Ipv6 => 128
+        }
+    }
 }
 
 
@@ -1769,10 +1884,7 @@ mod test {
     fn ip_block_from_v4_str() {
         fn check(s: &str, prefix: bool, min: &str, max: &str) {
             let block = IpBlock::from_v4_str(s).unwrap();
-            let is_prefix = match block {
-                IpBlock::Prefix(_) => true,
-                _ => false
-            };
+            let is_prefix = matches!(block, IpBlock::Prefix(_));
             assert_eq!(prefix, is_prefix);
             assert_eq!(
                 block.min(),
@@ -1827,10 +1939,7 @@ mod test {
     fn ip_block_from_str() {
         fn check_v4(s: &str, prefix: bool, min: &str, max: &str) {
             let block = IpBlock::from_str(s).unwrap();
-            let is_prefix = match block {
-                IpBlock::Prefix(_) => true,
-                _ => false
-            };
+            let is_prefix = matches!(block, IpBlock::Prefix(_));
             assert_eq!(prefix, is_prefix);
             assert_eq!(
                 block.min(),
@@ -1915,19 +2024,19 @@ mod test {
     #[test]
     fn addr_from() {
         assert_eq!(
-            Addr::from(0x12345678_12345678),
-            Addr(0x12345678_12345678)
+            Addr::from(0x1234_5678_1234_5678),
+            Addr(0x1234_5678_1234_5678)
         );
         assert_eq!(
-            u128::from(Addr(0x12345678_12345678)),
-            0x12345678_12345678
+            u128::from(Addr(0x1234_5678_1234_5678)),
+            0x1234_5678_1234_5678
         );
         assert_eq!(
             Addr::from(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1))),
-            Addr(0x7F000001_00000000_00000000_00000000)
+            Addr(0x7F00_0001_0000_0000_0000_0000_0000_0000)
         );
         assert_eq!(
-            Ipv4Addr::from(Addr(0x7F000001_00000000_00000000_00000000)),
+            Ipv4Addr::from(Addr(0x7F00_0001_0000_0000_0000_0000_0000_0000)),
             Ipv4Addr::new(127, 0, 0, 1)
         );
         assert_eq!(
@@ -1947,12 +2056,12 @@ mod test {
     #[test]
     fn addr_to_min_max() {
         assert_eq!(
-            Addr(0x12345678_12345678_12345678_12345678).to_min(11).0,
-            0x12200000_00000000_00000000_00000000
+            Addr(0x1234_5678_1234_5678_1234_5678_1234_5678).to_min(11).0,
+            0x1220_0000_0000_0000_0000_0000_0000_0000
         );
         assert_eq!(
-            Addr(0x12345678_12345678_12345678_12345678).to_max(11).0,
-            0x123fffff_ffffffff_ffffffff_ffffffff
+            Addr(0x1234_5678_1234_5678_1234_5678_1234_5678).to_max(11).0,
+            0x123f_ffff_ffff_ffff_ffff_ffff_ffff_ffff
         );
     }
 }
